package com.persagy.proxy.adm.controller;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.util.StrUtil;
import com.persagy.proxy.adm.constant.AdmCommonConstant;
import com.persagy.proxy.adm.model.AdmEquipment;
import com.persagy.proxy.adm.request.AdmCreateRequest;
import com.persagy.proxy.adm.request.AdmQueryCriteria;
import com.persagy.proxy.adm.request.AdmResponse;
import com.persagy.proxy.adm.service.IAdmEquipmentService;
import com.persagy.proxy.common.entity.InstanceUrlParam;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.web.bind.annotation.*;

import javax.ws.rs.QueryParam;
import java.util.List;

/**
 * Adm 楼层 API
 * @author lvxy
 * @date 2021/8/16
 */

@RestController
@RequestMapping("/object/equip")
public class AdmEquipmentController {
    @Autowired
    private IAdmEquipmentService service;
    @Value("${middleware.group.code}")
    private String groupCode;

    /**
     * 统计
     * @param projectId
     * @param request
     * @return
     * @throws Exception
     */
    @PostMapping("/count")
    public AdmResponse count(@QueryParam("projectId") String projectId,
                             @RequestBody AdmQueryCriteria request) throws Exception {
        // 组装上下文条件
        InstanceUrlParam context = new InstanceUrlParam(groupCode, null, projectId, AdmCommonConstant.APP_ID);
        request.setName(AdmEquipment.OBJ_TYPE);
        request.setOnlyCount(true);
        return service.doQuery(context, request, AdmEquipment.class);
    }

    /**
     * 创建
     * @param createRequest
     * @return
     * @throws Exception
     */
    @PostMapping("/create")
    public AdmResponse create(@RequestBody AdmCreateRequest<AdmEquipment> createRequest, @QueryParam("projectId") String projectId) throws Exception {
        // 组装上下文条件
        InstanceUrlParam context = new InstanceUrlParam(groupCode, null, projectId, AdmCommonConstant.APP_ID);
        List<AdmEquipment> vos = CollUtil.newArrayList(createRequest.getContent());
        vos = service.doInsertEuip(context, AdmEquipment.class, vos);
        return AdmResponse.success(vos);
    }

    /**
     * 根据ID删除
     * @param vos
     * @return
     * @throws Exception
     */
    @PostMapping("/delete")
    public AdmResponse delete(@QueryParam("projectId") String projectId, @RequestBody List<AdmEquipment> vos) throws Exception {
        // 组装上下文条件
        InstanceUrlParam context = new InstanceUrlParam(groupCode, null, projectId, AdmCommonConstant.APP_ID);
        service.doDelete(context, vos);
        return AdmResponse.success();
    }


    /**
     * 更新
     * @param createRequest
     * @return
     * @throws Exception
     */
    @PostMapping("/update")
    public AdmResponse update(@RequestBody AdmCreateRequest<AdmEquipment> createRequest, @QueryParam("projectId") String projectId) throws Exception {
        // 组装上下文条件
        InstanceUrlParam context = new InstanceUrlParam(groupCode, null, projectId, AdmCommonConstant.APP_ID);
        List<AdmEquipment> vos = CollUtil.newArrayList(createRequest.getContent());
        //vos = service.doUpdateEuip(context, AdmEquipment.class, vos);
        vos = service.doUpdate(context, AdmEquipment.class, vos);
        return AdmResponse.success(vos);
    }

    /**
     * 查询
     * @param projectId
     * @param request
     * @return
     * @throws Exception
     */
    @PostMapping("/query")
    public AdmResponse query(@QueryParam("projectId") String projectId,
                             @RequestBody AdmQueryCriteria request) throws Exception {
        // 组装上下文条件
        InstanceUrlParam context = new InstanceUrlParam(groupCode, null, projectId, AdmCommonConstant.APP_ID);
        request.setName(AdmEquipment.OBJ_TYPE);
        return service.doQuery(context, request, AdmEquipment.class);
    }

    /**
     * 设备绑定的动态信息点或静态信息点
     * @param admEquipment
     * @return
     * @throws Exception
     */
    @GetMapping("/export")
    public AdmResponse exportInfos(@RequestBody AdmCreateRequest<AdmEquipment> admEquipment, @QueryParam("projectId") String projectId) throws Exception {
        // 组装上下文条件
        InstanceUrlParam context = new InstanceUrlParam(groupCode, null, projectId, AdmCommonConstant.APP_ID);
        List<AdmEquipment> vos = CollUtil.newArrayList(admEquipment.getContent());
        vos = service.doUpdate(context, AdmEquipment.class, vos);
        return AdmResponse.success(vos);
    }


    /**
     * 设备上传excel识别静态信息点
     * @param admEquipment
     * @return
     * @throws Exception
     */
    @PostMapping("/import")
    public AdmResponse importInfos(@RequestBody AdmCreateRequest<AdmEquipment> admEquipment, @QueryParam("projectId") String projectId) throws Exception {
        // 组装上下文条件
        InstanceUrlParam context = new InstanceUrlParam(groupCode, null, projectId, AdmCommonConstant.APP_ID);
        List<AdmEquipment> vos = CollUtil.newArrayList(admEquipment.getContent());
        vos = service.doUpdate(context, AdmEquipment.class, vos);
        return AdmResponse.success(vos);
    }

    /**
     * 根据项目id查询所有的设备和部件
     * @param projectId
     * @param request
     * @return
     * @throws Exception
     */
    @GetMapping("/query-equip")
    public AdmResponse queryEquip(@QueryParam("projectId") String projectId,
                                  @RequestBody AdmQueryCriteria request) throws Exception {
        // 组装上下文条件
        InstanceUrlParam context = new InstanceUrlParam(groupCode, null, projectId, AdmCommonConstant.APP_ID);
        //request.setName("equipment");
        if(StrUtil.isNotEmpty(request.getFilters())){
            request.setFilters(String.join(";",request.getFilters(),"objType in ['equipment','component']"));
        }else{
            request.setFilters("objType in ['equipment','component']");
        }
        return service.doQuery(context, request, AdmEquipment.class);
    }

    /**
     * 批量创建设备同时创建资产、创建设备和系统的关系
     * @param admEquipment
     * @return
     * @throws Exception
     */
    @PostMapping("/create-equip-pro")
    public AdmResponse createExt(@RequestBody AdmCreateRequest<AdmEquipment> admEquipment, @QueryParam("projectId") String projectId) throws Exception {
        // 组装上下文条件
        InstanceUrlParam context = new InstanceUrlParam(groupCode, null, projectId, AdmCommonConstant.APP_ID);
        List<AdmEquipment> vos = CollUtil.newArrayList(admEquipment.getContent());
        vos = service.doInsertExt(context, AdmEquipment.class, vos);
        return AdmResponse.success(vos);
    }


}
