package com.persagy.proxy.adm.controller;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.util.StrUtil;
import com.persagy.dmp.common.model.entity.BaseEntity;
import com.persagy.proxy.adm.constant.AdmCommonConstant;
import com.persagy.proxy.adm.model.AdmEquipment;
import com.persagy.proxy.adm.model.AdmFloor;
import com.persagy.proxy.adm.model.AdmISpace;
import com.persagy.proxy.adm.model.AdmSpace;
import com.persagy.proxy.adm.request.AdmCreateRequest;
import com.persagy.proxy.adm.request.AdmQueryCriteria;
import com.persagy.proxy.adm.request.AdmResponse;
import com.persagy.proxy.adm.service.IAdmEquipmentService;
import com.persagy.proxy.adm.service.IAdmFloorService;
import com.persagy.proxy.adm.service.IAdmISpaceService;
import com.persagy.proxy.adm.service.IAdmSpaceService;
import com.persagy.proxy.adm.utils.ObjectNameUtil;
import com.persagy.proxy.common.entity.DmpResult;
import com.persagy.proxy.common.entity.InstanceUrlParam;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.ws.rs.QueryParam;
import java.util.List;
import java.util.Map;

/**
 * Adm 楼层 API
 * @author lvxy
 * @date 2021/8/16
 */

@RestController
@RequestMapping("/object/floor")
public class AdmFloorController {
    @Autowired
    private IAdmFloorService service;
    @Autowired
    private IAdmEquipmentService equipmentService;
    @Autowired
    private IAdmISpaceService iAdmISpaceService;
    @Autowired
    private IAdmSpaceService spaceService;
    @Value("${middleware.group.code}")
    private String groupCode;

    /**
     * 创建
     * @param createRequest
     * @return
     * @throws Exception
     */
    @PostMapping("/create")
    public AdmResponse create(@RequestBody AdmCreateRequest<AdmFloor> createRequest, @QueryParam("projectId") String projectId) throws Exception {
        // 组装上下文条件
        InstanceUrlParam context = new InstanceUrlParam(groupCode, null, projectId, AdmCommonConstant.APP_ID);
        List<AdmFloor> vos = createRequest.getContent();
        for(AdmFloor floor : vos){
            if(StrUtil.isEmpty(floor.getName())){
                floor.setName(ObjectNameUtil.objectName("楼层-"));
            }
            if(floor.getFloorSequenceID() == null){
                return AdmResponse.failure("楼层顺序号不可以为空或类型错误");
            }
        }
        Map<String, AdmFloor> admFloorMap = CollUtil.fieldValueMap(vos, "buildingId");
        if (CollUtil.isNotEmpty(admFloorMap)){
            AdmQueryCriteria admQueryCriteria = new AdmQueryCriteria();
            StringBuilder stringBuilder = new StringBuilder();
            stringBuilder.append("buildingId in [");
            admFloorMap.keySet().stream().forEach(buildingId -> {
                stringBuilder.append("'").append(buildingId).append("'").append(",");
            });
            String filters = StrUtil.sub(stringBuilder.toString(),0, -1);
            filters+= "]";
            admQueryCriteria.setFilters(filters);
            admQueryCriteria.setName("floor");
            AdmResponse admResponseF = service.doQuery(context, admQueryCriteria, AdmFloor.class);
            if(DmpResult.SUCCESS.equals(admResponseF.getResult()) && admResponseF.getTotal() > 0){
                List<AdmFloor> vosOld = (List<AdmFloor>) admResponseF.getContent();
                for(AdmFloor admFloorOld : vosOld){
                    if(admFloorMap.containsKey(admFloorOld.getBuildingId())){
                        AdmFloor admFloorNew = admFloorMap.get(admFloorOld.getBuildingId());
                        //楼层序列号相同
                        if(admFloorOld.getFloorSequenceID() !=null && admFloorNew.getFloorSequenceID().equals(admFloorOld.getFloorSequenceID())){
                           return AdmResponse.failure("楼层顺序号不可以重复");
                        }
                    }
                }
            }
        }
        List<AdmFloor> admFloors = service.doInsert(context, AdmFloor.class, createRequest.getContent());
        return AdmResponse.success(admFloors);
    }

    /**
     * 根据ID删除
     * @param vos
     * @return
     * @throws Exception
     */
    @PostMapping("/delete")
    public AdmResponse delete(@QueryParam("projectId") String projectId, @RequestBody List<AdmFloor> vos) throws Exception {
        // 组装上下文条件
        InstanceUrlParam context = new InstanceUrlParam(groupCode, null, projectId, AdmCommonConstant.APP_ID);
        String msg = "楼层下包含其他对象实例，请解除关系后删除";
        Map<String, AdmFloor> admFloorMap = CollUtil.fieldValueMap(vos, BaseEntity.PROP_ID);

        AdmQueryCriteria admQueryCriteria = new AdmQueryCriteria();
        StringBuilder stringBuilder = new StringBuilder();
        stringBuilder.append("floorId in [");
        admFloorMap.keySet().stream().forEach(buildingId -> {
            stringBuilder.append("'").append(buildingId).append("'").append(",");
        });
        String filters = StrUtil.sub(stringBuilder.toString(),0, -1);
        filters+= "]";

        admQueryCriteria.setFilters(filters);
        admQueryCriteria.setOnlyCount(true);

        admQueryCriteria.setName(AdmSpace.OBJ_TYPE);
        AdmResponse admResponseSpace = spaceService.doQuery(context, admQueryCriteria, AdmSpace.class);
        if(admResponseSpace.getResult().equals(DmpResult.SUCCESS) && CollUtil.isNotEmpty(admResponseSpace.getContent())){
            return AdmResponse.failure(msg);
        }
        admQueryCriteria.setName(AdmISpace.OBJ_TYPE);
        AdmResponse admResponseISpace = iAdmISpaceService.doQuery(context, admQueryCriteria, AdmISpace.class);
        if(admResponseISpace.getResult().equals(DmpResult.SUCCESS) && CollUtil.isNotEmpty(admResponseISpace.getContent())){
            return AdmResponse.failure(msg);
        }
        admQueryCriteria.setName(AdmEquipment.OBJ_TYPE);
        AdmResponse admResponseEuip = equipmentService.doQuery(context, admQueryCriteria, AdmEquipment.class);
        if(admResponseEuip.getResult().equals(DmpResult.SUCCESS) && CollUtil.isNotEmpty(admResponseEuip.getContent())){
            return AdmResponse.failure(msg);
        }
        service.doDelete(context, vos);
        return AdmResponse.success();
    }

    /**
     * 根据ID删除 - 单条删除
     * @param vos
     * @return
     * @throws Exception
     */
    @PostMapping("/single/delete")
    public AdmResponse deleteSingle(@QueryParam("projectId") String projectId, @RequestBody List<AdmFloor> vos) throws Exception {
       delete(projectId,vos);
       return AdmResponse.success();
    }

    /**
     * 更新
     * @param admFloor
     * @return
     * @throws Exception
     */
    @PostMapping("/update")
    public AdmResponse update(@RequestBody AdmCreateRequest<AdmFloor> admFloor, @QueryParam("projectId") String projectId) throws Exception {
        // 组装上下文条件
        InstanceUrlParam context = new InstanceUrlParam(groupCode, null, projectId, AdmCommonConstant.APP_ID);
        List<AdmFloor> vos = CollUtil.newArrayList(admFloor.getContent());
        /** 遍历楼层列表 */
        Map<String, AdmFloor> admFloorBdMap = CollUtil.fieldValueMap(vos, "buildingId");
        if (CollUtil.isNotEmpty(admFloorBdMap)){
            AdmQueryCriteria admQueryCriteria = new AdmQueryCriteria();
            StringBuilder stringBuilder = new StringBuilder();
            stringBuilder.append("buildingId in [");
            admFloorBdMap.keySet().stream().forEach(buildingId -> {
                stringBuilder.append("'").append(buildingId).append("'").append(",");
            });
            String filters = StrUtil.sub(stringBuilder.toString(),0, -1);
            filters+= "]";
            admQueryCriteria.setFilters(filters);
            admQueryCriteria.setName("floor");
            AdmResponse admResponseF = service.doQuery(context, admQueryCriteria, AdmFloor.class);
            if(DmpResult.SUCCESS.equals(admResponseF.getResult()) && admResponseF.getTotal() > 0 && CollUtil.isNotEmpty(admResponseF.getContent())){
                List<AdmFloor> vosOld = (List<AdmFloor>) admResponseF.getContent();
                for(AdmFloor admFloorOld : vosOld){
                    if(admFloorBdMap.containsKey(admFloorOld.getBuildingId())){
                        AdmFloor admFloorNew = admFloorBdMap.get(admFloorOld.getBuildingId());
                        //楼层序列号相同
                        if(!admFloorOld.getId().equals(admFloorNew.getId()) && admFloorOld.getFloorSequenceID()!=null && admFloorNew.getFloorSequenceID().equals(admFloorOld.getFloorSequenceID())){
                            return AdmResponse.failure("楼层顺序号不可以重复");
                        }
                    }
                }
            }
        }
        vos = service.doUpdate(context, AdmFloor.class, vos);
        return AdmResponse.success(vos);
    }

    /**
     * 更新 outline
     * TODO 暂时不知道具体的参数，先做所有字段的更新，之后有具体场景再做细化
     * @param admFloor
     * @return
     * @throws Exception
     */
    @PostMapping("/update-outline")
    public AdmResponse updateOutline(@RequestBody AdmCreateRequest<AdmFloor> admFloor, @QueryParam("projectId") String projectId) throws Exception {
        // 组装上下文条件
        AdmResponse admResponse = update(admFloor,projectId);
        //TODO 处理关系
        /** 处理关系标记 */
       /* val listtype = ArrayList<String>()
        listtype.add("sh2bd")
        listtype.add("bd2sp")
        listtype.add("fl2sp")
        listtype.add("syeq2bd")
        listtype.add("sy2fl")
        listtype.add("eq2fl")
        listtype.add("pe2bd")
        listtype.add("pe2fl")
        for (type in listtype){
            val entity = EquipmentService.relationTypeProjectservice.select(SFilter.eq("projectId", Opt.projectId!!), SFilter.eq("relationType", type)).entity()
            if (entity!=null){
                entity.computationalState = 2
                EquipmentService.relationTypeProjectservice.update(entity)
            }
        }*/
        return admResponse;
    }


    /**
     * 查询
     * @param projectId
     * @param request
     * @return
     * @throws Exception
     */
    @PostMapping("/query")
    public AdmResponse query(@QueryParam("projectId") String projectId,
                             @RequestBody AdmQueryCriteria request) throws Exception {
        // 组装上下文条件
        InstanceUrlParam context = new InstanceUrlParam(groupCode, null, projectId, AdmCommonConstant.APP_ID);
        request.setName(AdmFloor.OBJ_TYPE);
        return service.doQuery(context, request, AdmFloor.class);
    }


    /**
     * 查询
     * @param projectId
     * @param request
     * @return
     * @throws Exception
     */
    @PostMapping("/query-outline")
    public AdmResponse queryOutline(@QueryParam("projectId") String projectId,
                             @RequestBody AdmQueryCriteria request) throws Exception {
        // 组装上下文条件
        InstanceUrlParam context = new InstanceUrlParam(groupCode, null, projectId, AdmCommonConstant.APP_ID);
        request.setName(AdmFloor.OBJ_TYPE);
        return service.doQueryOutLine(context, request, AdmFloor.class);
    }

}
