package com.persagy.proxy.adm.controller;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.net.URLEncoder;
import java.nio.charset.StandardCharsets;
import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.poi.xssf.usermodel.XSSFCell;
import org.apache.poi.xssf.usermodel.XSSFRow;
import org.apache.poi.xssf.usermodel.XSSFSheet;
import org.apache.poi.xssf.usermodel.XSSFWorkbook;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.util.ResourceUtils;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;
import org.springframework.web.multipart.MultipartFile;

import com.alibaba.excel.EasyExcel;
import com.alibaba.excel.ExcelWriter;
import com.alibaba.fastjson.JSONObject;
import com.fasterxml.jackson.databind.node.ObjectNode;
import com.google.common.collect.Lists;
import com.persagy.dmp.define.client.DigitalGraphDefineFacade;
import com.persagy.proxy.adm.constant.AdmCommonConstant;
import com.persagy.proxy.adm.model.AdmRelationObject;
import com.persagy.proxy.adm.request.AdmQueryCriteria;
import com.persagy.proxy.adm.request.AdmResponse;
import com.persagy.proxy.adm.service.IRelationReportService;
import com.persagy.proxy.adm.strategy.RelationObjectContext;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.util.StrUtil;
import lombok.extern.slf4j.Slf4j;

/**
 * 报表/模板下载
 * 
 * @version 1.0.0
 * @company persagy
 * @author zhangqiankun
 * @date 2021年8月30日 下午7:41:56
 */
@Slf4j
@RestController
public class RelationReportController {

	public static final String EXPORT_SHEET_NAME = "关系维护";
	
	@Autowired
	private RelationObjectContext relationObjectContext;
	
	@Autowired
	private IRelationReportService relationReportService;
	
	
	/**
	 * 查询总数量
	 * 
	 * @param queryCriteria
	 * @param request
	 * @return
	 */
	@RequestMapping("/graphic/query-count")
	public AdmResponse queryCount(HttpServletRequest request) {
		String groupCode = request.getHeader(AdmCommonConstant.GROUP_CODE_HEADER);
		String projectId = request.getHeader(AdmCommonConstant.PROJECT_ID_HEADER);
		
		JSONObject result = new JSONObject();
		result.put("count", 0);
		return AdmResponse.success(Lists.newArrayList(result));
	}
	
	/**
	 * 查询总览数量-根据类型查询关系数量
	 * 
	 * @param type	relationType
	 */
	@RequestMapping("/graphic/type/query-count")
	public AdmResponse countRelationObjects(@RequestParam String type, HttpServletRequest request) {
		String groupCode = request.getHeader(AdmCommonConstant.GROUP_CODE_HEADER);
		String projectId = request.getHeader(AdmCommonConstant.PROJECT_ID_HEADER);
		ObjectNode objectNode = this.relationReportService.getRelationProjectCal(groupCode, projectId, type);
 		return AdmResponse.success(Lists.newArrayList(objectNode));
	}
	
	/**
	 * 项目关系类型查询
	 * 
	 * @param queryCriteria
	 * @param request
	 * @return
	 */
	@RequestMapping("/graphic/relation_type_project")
	public AdmResponse queryRelationObjects(@RequestParam AdmQueryCriteria queryCriteria, HttpServletRequest request) {
		String groupCode = request.getHeader(AdmCommonConstant.GROUP_CODE_HEADER);
		String projectId = request.getHeader(AdmCommonConstant.PROJECT_ID_HEADER);
		JSONObject result = new JSONObject();
		result.put("count", 0);
		return AdmResponse.success(Lists.newArrayList(result));
	}
	
	
	/**
	 * 下载报告-设备静态信息维护模板
	 * 
	 * @param request
	 * @param response
	 */
	@RequestMapping("/graphic/downloads/equip/template")
	public void downloadEquipTemplate(HttpServletRequest request, HttpServletResponse response) {
		ExcelWriter excelWriter = null;
		try {
			response.setContentType("application/vnd.ms-excel");
			String encode = StandardCharsets.UTF_8.name();
			response.setCharacterEncoding(encode);
			// 防止中文乱码
			String fileName = URLEncoder.encode("台账信息导入模板", encode);
			response.setHeader("Content-disposition", "attachment;filename=" + fileName + ".xlsx");
			
			File templateFile = this.getTemplateFile("equip_template.xlsx");
			excelWriter = EasyExcel.write(response.getOutputStream()).withTemplate(templateFile).build();
		} catch (Exception e) {
			log.error("设备静态信息维护模板下载失败", e);
		} finally {
			if (excelWriter != null) {
				excelWriter.finish();
			}
		}
	}

	/**
	 * 下载报告-点表
	 * 
	 * @param request
	 * @param response
	 */
	@RequestMapping("/graphic/downloads/point")
	public void downloadPoint(@RequestParam(required = false) String key, HttpServletRequest request, HttpServletResponse response) {
		ExcelWriter excelWriter = null;
		try {
			response.setContentType("application/vnd.ms-excel");
			String encode = StandardCharsets.UTF_8.name();
			response.setCharacterEncoding(encode);
			response.setHeader("Content-disposition", "attachment;filename=" + key);
			
			File file = null;
			try {
				file = ResourceUtils.getFile(ResourceUtils.CLASSPATH_URL_PREFIX + "template/upload/" + key);
			} catch (FileNotFoundException e) {
				file = ResourceUtils.getFile(AdmCommonConstant.SERVER_ROOT_PATH + "template/upload/" + key);
			}
			if (file == null || !file.exists()) {
				try {
					file = ResourceUtils.getFile(ResourceUtils.CLASSPATH_URL_PREFIX + "/template/relation-template.xlsx");
				} catch (FileNotFoundException e) {
					file = ResourceUtils.getFile(AdmCommonConstant.SERVER_ROOT_PATH + "/template/relation-template.xlsx");
				}
			}
			
			excelWriter = EasyExcel.write(response.getOutputStream()).withTemplate(file).build();
		} catch (Exception e) {
			log.error("下载报告-点表失败", e);
		} finally {
			if (excelWriter != null) {
				excelWriter.finish();
			}
		}
	}
	
	/**
	 * 下载模板
	 * 
	 * @param request
	 * @param response
	 */
	@RequestMapping("/graphic/template-downloads")
	public void templateDownloads(HttpServletRequest request, HttpServletResponse response) {
		ExcelWriter excelWriter = null;
		try {
			response.setContentType("application/vnd.ms-excel");
			String encode = StandardCharsets.UTF_8.name();
			response.setCharacterEncoding(encode);
			response.setHeader("Content-disposition", "attachment;filename=relation-template.xlsx");
			
			File templateFile = this.getTemplateFile("relation-template.xlsx");
			excelWriter = EasyExcel.write(response.getOutputStream()).withTemplate(templateFile).build();
		} catch (Exception e) {
			log.error("模板下载失败", e);
		} finally {
			if (excelWriter != null) {
				excelWriter.finish();
			}
		}
	}
	
	/**
	 * 下载报告
	 * 
	 * @param request
	 * @param response
	 */
	@RequestMapping("/graphic/report-downloads")
	public void reportDownloads(@RequestParam String projectId, @RequestParam String relType, @RequestParam(required = false) String zoneType, 
			HttpServletRequest request, HttpServletResponse response) {
		ExcelWriter excelWriter = null;
		try {
			response.setContentType("application/vnd.ms-excel");
			String encode = StandardCharsets.UTF_8.name();
			response.setCharacterEncoding(encode);
			
			String templateFile = projectId + AdmCommonConstant.LINE_THROUGH + relType + AdmCommonConstant.EXCEL_SUFFIX_XLSX;
			response.setHeader("Content-disposition", "attachment;filename=" + templateFile);
			
			File file = this.getTemplateOrDefaultFile(templateFile);
			excelWriter = EasyExcel.write(response.getOutputStream()).withTemplate(file).build();
		} catch (Exception e) {
			log.error("下载报告失败", e);
		} finally {
			if (excelWriter != null) {
				excelWriter.finish();
			}
		}
	}
	
	/**
	 * 导出-点位模板
	 * 
	 * @param request
	 * @param response
	 */
	@RequestMapping("/graphic/export/point")
	public void exportPoint(@RequestParam String buildingId, @RequestParam String floorId, @RequestParam String category, 
			HttpServletRequest request, HttpServletResponse response) {
		ExcelWriter excelWriter = null;
		try {
			response.setContentType("application/vnd.ms-excel");
			String encode = StandardCharsets.UTF_8.name();
			response.setCharacterEncoding(encode);
			
			//excelWriter = EasyExcel.write(response.getOutputStream()).withTemplate(file).build();
		} catch (Exception e) {
			log.error("导出-点位模板失败", e);
		} finally {
			if (excelWriter != null) {
				excelWriter.finish();
			}
		}
	}
	
	/**
	 * 导出关系
	 * 
	 * @param request
	 * @param response
	 */
	@RequestMapping("/graphic/export")
	public void export(@RequestParam String projectId, @RequestParam String relType, @RequestParam(required = false) String code, 
			@RequestParam(required = false) String zoneType, HttpServletRequest request, HttpServletResponse response) {
		ExcelWriter excelWriter = null;
		try {
			String groupCode = request.getHeader(AdmCommonConstant.GROUP_CODE_HEADER);
			code = StrUtil.isBlank(code) ? "对象ID" : code;
			
			response.setContentType("application/vnd.ms-excel");
			String encode = StandardCharsets.UTF_8.name();
			response.setCharacterEncoding(encode);
			response.setHeader("Content-disposition", "attachment;filename=relation-template.xlsx");
			
			List<AdmRelationObject> relationObjects = this.relationReportService.reportRelationObject(groupCode, projectId, relType, code, zoneType);
			
			File templateFile = this.getTemplateFile("relation-template.xlsx");
			EasyExcel.write(response.getOutputStream()).withTemplate(templateFile).sheet(EXPORT_SHEET_NAME).needHead(false).doWrite(relationObjects);
		} catch (Exception e) {
			log.error("导出关系失败", e);
		} finally {
			if (excelWriter != null) {
				excelWriter.finish();
			}
		}
	}
	
	
	/************************************************** 文件上传 ***********************************************/
	
	/**
	 * 文件导入，这里由于需要回写文件，且存储到本地，采用原始POI文件解析、上传、回写、存储
	 * 
	 * @param relType
	 * @param code
	 * @param zoneType
	 * @param request
	 * @param response
	 */
	@SuppressWarnings("resource")
	@RequestMapping("/graphic/import")
	public AdmResponse importFile(@RequestParam("file") MultipartFile file, @RequestParam String relType, @RequestParam(required = false) String zoneType, 
			HttpServletRequest request, HttpServletResponse response) {
		String failure = null;
		FileOutputStream fileOutputStream = null;
		try {
			if (file == null) {
				return AdmResponse.failure("未选择文件");
			}
			
			int failurNum = 0;
			int successNum = 0;
			String graphId = null;
			// 临时存储行号
			Set<Integer> rowNums = new HashSet<Integer>();
			// 批量更新数据
			List<ObjectNode> relationObjects = new ArrayList<ObjectNode>();
			
			String groupCode = request.getHeader(AdmCommonConstant.GROUP_CODE_HEADER);
			String projectId = request.getHeader(AdmCommonConstant.PROJECT_ID_HEADER);
			
			XSSFWorkbook workbook = new XSSFWorkbook(file.getInputStream());
		    XSSFSheet sheet = workbook.getSheet("关系维护");
		    // 获取识别码的属性
		    String code = sheet.getRow(2).getCell(1).toString();
		    int lastRowNum = sheet.getLastRowNum();
		    for (int i = 5; i <= lastRowNum; i++) {
		    	XSSFRow xssfRow = sheet.getRow(i);
		    	if (xssfRow == null) {
					continue;
				}
		    	// 封装cell数据
		    	XSSFCell masterCodeCell = xssfRow.getCell(0);
		    	XSSFCell slaveCodeCell = xssfRow.getCell(8);
		    	XSSFCell masterIdCell = xssfRow.getCell(3);
		    	XSSFCell slaveIdCell = xssfRow.getCell(11);
		    	
		    	AdmRelationObject relationObject = new AdmRelationObject();
		    	relationObject.setMasterCode(masterCodeCell == null ? null : masterCodeCell.getStringCellValue());
		    	relationObject.setSlaveCode(slaveCodeCell == null ? null : slaveCodeCell.getStringCellValue());
		    	relationObject.setMasterId(masterIdCell == null ? null : masterIdCell.getStringCellValue());
		    	relationObject.setSlaveId(slaveIdCell == null ? null : slaveIdCell.getStringCellValue());
		    	
		    	// 校验数据
				Object result = this.relationObjectContext.checkRelationObject(relationObject, groupCode, projectId, relType, code);
				if (result instanceof String) {
					failurNum++;
					xssfRow.createCell(16).setCellValue((String)result);
					continue;
				}
				ObjectNode relation = (ObjectNode) result;
				graphId = relation.get("graphId") == null ? graphId : relation.get("graphId").asText();
				
				// 每一千条数据，批量新增一次
				rowNums.add(i);
				relationObjects.add(relation);
				if (relationObjects.size() >= 1000) {
					boolean objects = this.relationObjectContext.saveRelationObjects(relationObjects, groupCode, projectId, relType);
					if (objects) {
						successNum += relationObjects.size();
					} else {
						failurNum += relationObjects.size();
						for (int rowNum : rowNums) {
							sheet.getRow(rowNum).createCell(16).setCellValue("调用中台，存储失败");
						}
					}
					rowNums.clear();
					relationObjects.clear();
				}
			}
		    // 最后一次遗留数据录入
		    if (CollectionUtil.isNotEmpty(relationObjects)) {
		    	boolean objects = this.relationObjectContext.saveRelationObjects(relationObjects, groupCode, projectId, relType);
				if (objects) {
					successNum += relationObjects.size();
				} else {
					failurNum += relationObjects.size();
					for (int rowNum : rowNums) {
						sheet.getRow(rowNum).createCell(16).setCellValue("调用中台，存储失败");
					}
				}
			}
		    
        	try {
        		// 调用中台存储完成之后，此文件存储至本地
        		String uploadFile = projectId + AdmCommonConstant.LINE_THROUGH + relType + AdmCommonConstant.EXCEL_SUFFIX_XLSX;
        		File templateFile = this.createTemplateFile(uploadFile);
        		fileOutputStream = new FileOutputStream(templateFile);
        		workbook.write(fileOutputStream);
			} catch (Exception e) {
				log.error("文件存储失败", e);
			}
        	
        	return AdmResponse.success(Lists.newArrayList(this.getImportResult(graphId, successNum, failurNum)));
		} catch (Exception e) {
			log.error("文件导入失败", e);
			failure = e.getMessage();
		} finally {
			if (fileOutputStream != null) {
				try {
					fileOutputStream.flush();
					fileOutputStream.close();
				} catch (IOException e) {}
			}
		}
		return AdmResponse.failure(failure);
	}
	
	
	
	
	/************************************************** 本地文件获取、创建 ***********************************************/
	
	
	/**
	 * 获取对应的文件，或者默认文件
	 * 不同路径，防止镜像部署时获取不到
	 * 
	 * @param templateName 带后缀的文件名
	 * @return
	 * @throws FileNotFoundException 
	 */
	private File getTemplateOrDefaultFile(String templateName) throws FileNotFoundException {
		File file = null;
		try {
			file = ResourceUtils.getFile(ResourceUtils.CLASSPATH_URL_PREFIX + "template/" + templateName);
		} catch (FileNotFoundException e) {
			file = ResourceUtils.getFile(AdmCommonConstant.SERVER_ROOT_PATH + "template/" + templateName);
		}
		if (file == null || !file.exists()) {
			try {
				file = ResourceUtils.getFile(ResourceUtils.CLASSPATH_URL_PREFIX + "template/relation-template.xlsx");
			} catch (FileNotFoundException e) {
				file = ResourceUtils.getFile(AdmCommonConstant.SERVER_ROOT_PATH + "/template/relation-template.xlsx");
			}
		}
		return file;
	}
	
	/**
	 * 获取对应的模板文件，不同路径，防止镜像内获取不到
	 * 
	 * @param templateName 带后缀的文件名
	 * @return
	 * @throws FileNotFoundException 
	 */
	private File getTemplateFile(String templateName) throws FileNotFoundException {
		File file = null;
		try {
			file = ResourceUtils.getFile(ResourceUtils.CLASSPATH_URL_PREFIX + "template/" + templateName);
		} catch (FileNotFoundException e) {}
		if (file == null || !file.exists()) {
			file = ResourceUtils.getFile(AdmCommonConstant.SERVER_ROOT_PATH + "/template/" + templateName);
		}
		return file;
	}
	
	/**
	 * 创建本地文件，这里不再在classpath下创建
	 * 
	 * @param uploadFile 文件名称，带后缀
	 * @throws IOException 
	 */
	private File createTemplateFile(String uploadFile) throws IOException {
		String path = AdmCommonConstant.SERVER_ROOT_PATH + "/template";
		File file = new File(path);
		if (!file.exists()) {
			file.mkdirs();
		}
		file = new File(path + "/" + uploadFile);
		if (!file.exists()) {
			file.createNewFile();
		}
		return file;
	}
	
	/**
	 * 获取导入结果
	 * 
	 * @return
	 */
	private JSONObject getImportResult(String graphId, int successNum, int failureNum) {
		JSONObject result = new JSONObject();
		result.put("relationType", graphId);
		result.put("successCount", successNum);
		result.put("failCount", failureNum);
		String state = null;
        if (successNum > 0 && failureNum > 0) {
        	state = "1";
        } else if (successNum > 0 && failureNum == 0) {
            state = "0";
        } else {
            state = "2";
        }
        result.put("state", state);
		return result;
	}

}
