package com.persagy.proxy.common.handler;

import cn.hutool.core.util.StrUtil;
import com.persagy.dmp.common.constant.CommonConstant;
import com.persagy.dmp.common.context.AppContext;
import org.springframework.web.servlet.ModelAndView;
import org.springframework.web.servlet.handler.HandlerInterceptorAdapter;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

/**
 * 上下文 拦截器
 * @author Charlie Yu
 * @date 2021-09-13
 */
public class AdmContextHandler extends HandlerInterceptorAdapter {

    @Override
    public boolean preHandle(HttpServletRequest request, HttpServletResponse response, Object handler) throws Exception {
        // 从header中获取上下文
        if(StrUtil.isNotBlank(request.getHeader("groupCode"))) {
            // 请求参数中包含groupCode，由于没有标识位，暂时用这种方式判断吧
            accessRequestInfo(request);
        } else {
            // 其他情况，按现有系统默认不验证的逻辑，其他情况就是默认放行，不必处理。最好是抛401
        }
        return true;
    }

    /**
     * 从请求参数中获取上下文
     * @param request
     */
    private void accessRequestInfo(HttpServletRequest request) {
        AppContext.getContext().setGroupCode(request.getHeader("groupCode"));
        AppContext.getContext().setProjectId(request.getHeader("projectId"));
        AppContext.getContext().setAppId(request.getHeader("appId"));
        String userId = request.getHeader("userId");
        // 无用户时，默认为默认系统用户
        if(StrUtil.isBlank(userId)) {
            userId = CommonConstant.DEFAULT_ID;
        }
        AppContext.getContext().setAccountId(userId);
    }

    @Override
    public void postHandle(HttpServletRequest request, HttpServletResponse response, Object handler, ModelAndView modelAndView) throws Exception {
        AppContext.unload();
    }

    @Override
    public void afterCompletion(HttpServletRequest request, HttpServletResponse response, Object handler, Exception ex) throws Exception {
        AppContext.unload();
    }
}
